# Test program to ask about destructors and PyQt4.

import sys, time
from PyQt4.QtGui import QApplication
from PyQt4.QtCore import QThread, QTimer, QObject

class Worker(QObject):
    def run(self):
        time.sleep(1)

class ThreadController(QObject):
    def __init__(self, parent):
        QObject.__init__(self, parent)
        # No parent object provided; if so, we get the error
        # "QObject::moveToThread: Cannot move objects with a parent".
        # Who will deallocate this object, since it has no parent? I
        # assume the answer is "Python, maybe". Is that good enough?
        self.worker = Worker()
        self.workerThread = QThread(self)
        self.worker.moveToThread(self.workerThread)
        # Use method #2 below. Commnet out for a crash.
        parent.aboutToQuit.connect(self.del_)
        self.workerThread.start()

    # I'd like to run when when ~ThreadController is invoked. That's quite
    # differentfrom when __del__ is invoked. How?
    #
    # 1. Give up. Just invoke it manually. However, this is error-prone (i.e.
    #    I'll forget to do it at some point).
    # 2. Connect app.aboutToQuit to this. This seems fairly reasonable, but
    #    it would feel cleaner to invoke when the destructor is invoked,
    #    rather than earlier. Also, this means that the QApplication would
    #    need to be passed to every thread, which might involve awkwardness
    #    in passing it through several intervening subclasses.
    # 3. Connect destroyed(), which doesn't work -- this isn't emitted until
    #    the object is mostly dead, far after ~ThreadController was invoked,
    #    and won't be executed.
    # 4. Have __del__ invoke this, which doesn't work -- it only happens after
    #    ~ThreadController has finihed.
    def del_(self):
        print('del_')
        self.workerThread.quit()
        self.workerThread.wait()

    def __del__(self):
        print('__del__')
        # Crash when uncommented.
        #self.del_()

if __name__ == '__main__':
    app = QApplication(sys.argv)

    # Exit the program shortly after the event loop starts up, but before the
    # thread finishes.
    QTimer.singleShot(200, app.exit)

    # Start a thread
    tm = ThreadController(app)

    # Run the main event loop.
    ret = app.exec_()
    print('done')
    sys.exit(ret)
